from fabric.connection import Connection
from invoke.watchers import Responder
from patchwork.files import directory, exists


DEFAULT_KEY_DIRECTORY = '/var/deploykeys'


def get_keyfile(project_name, directory=DEFAULT_KEY_DIRECTORY, public=True):
    sanitized_name = project_name.replace('/', '_')
    suffix = '.pub' if public else ''
    keyfile = '{0}/{1}{2}'.format(directory, sanitized_name, suffix)
    return keyfile


def create_key(conn: Connection, project_name, owning_group, path=DEFAULT_KEY_DIRECTORY):
    directory(conn, path, group=owning_group, mode='600', sudo=True)
    keyfile = get_keyfile(project_name, path, public=False)
    if exists(conn, keyfile, sudo=True):
        conn.sudo('rm {0}'.format(keyfile))
    # Automatically provide an empty passphrase
    responder = Responder(r'passphrase.*:', '\n')
    conn.sudo('ssh-keygen -t rsa -b 4096 -C "{0}" -f {1}'.format(project_name, keyfile),
              watchers=[responder])



import setuptools

description = "Helper library for Fabric to simplify creating and managing GitHub deploy keys when deploying " \
              "GitHub-hosted repositories"

long_description = open('README.rst').read()

setuptools.setup(name='plush-fabric',
                 version='0.5.2',
                 description=description,
                 long_description=long_description,
                 author='Kevin Barnes',
                 author_email='kbarnes3@gmail.com',
                 url='https://github.com/kbarnes3/Plush',
                 license='BSD',
                 classifiers=[
                     'Development Status :: 4 - Beta',
                     'Intended Audience :: Developers',
                     'Topic :: Software Development',
                     'License :: OSI Approved :: MIT License',
                     "Operating System :: OS Independent",
                     'Programming Language :: Python :: 3',
                     'Programming Language :: Python :: 3.6',
                 ],
                 keywords='fabric github deployment',
                 install_requires=['oauth2client>=3.0.0', 'PyGithub>=1.3.8', 'keyring>=12.0.0', 'fabric>=2.5.0',
                                   'patchwork>=1.0.1'],
                 packages=['plush', 'plush.fabric_commands'],
                 entry_points={
                     'console_scripts': [
                         'auth = plush.console:auth_entry',
                         'listkeys = plush.console:list_keys_entry'
                     ]
                 },
                 )



from typing import Iterable

from fabric.connection import Connection
from invoke import UnexpectedExit
from invoke.watchers import Responder
from patchwork.files import exists

from .ssh_key import get_keyfile


__all__ = ['prepare_user', 'add_authorized_key', 'install_packages', 'get_keyfile']


def prepare_user(conn: Connection, user: str, group: str, add_sudo=True, no_sudo_passwd=False):
    messages = ''
    user_exists = False

    try:
        conn.sudo('getent passwd {0}'.format(user))
        user_exists = True
    except UnexpectedExit:
        pass

    if not user_exists:
        # Handle all the prompts for information about the new user like their name and room number
        responder = Responder(r'.*\[.*\].*', '\n')
        conn.sudo('adduser --disabled-password {0}'.format(user), pty=True, watchers=[responder])

    group_exists = False
    try:
        conn.sudo('getent group {0}'.format(group))
        group_exists = True
    except UnexpectedExit:
        pass

    if not group_exists:
        conn.sudo('addgroup {0}'.format(group))

    conn.sudo('adduser {0} {1}'.format(user, group))

    if add_sudo:
        conn.sudo('adduser {0} sudo'.format(user))

    if no_sudo_passwd:
        sudoers_file = '/etc/sudoers.d/{0}-plush'.format(user)
        if exists(conn, sudoers_file, sudo=True):
            conn.sudo('rm {0}'.format(sudoers_file))
        conn.sudo("echo '{0} ALL=(ALL:ALL) NOPASSWD:ALL' | sudo EDITOR='tee -a' visudo -f {1}"
                  .format(user, sudoers_file), pty=True)

    if add_sudo and not no_sudo_passwd:
        messages += 'Ensure {0} has a secure password configured ' \
                    'in order to run commands as sudo.\n'.format(user)
        messages += 'Alternatively, rerun this command with ' \
                    'the no_sudo_passwd=True parameter\n'
    if not user_exists:
        messages += 'The {0} account was created without a password set. ' \
                    'If a password is needed, a superuser will need to manually run: \n' \
                    'sudo passwd {0}\n' \
                    'Where possible, consider not setting a password and ' \
                    'instead using key based authentication.\n'.format(user)

    if messages:
        # Every message line ends in a newline.
        # If we've got a non-empty message, strip the last character
        messages = messages[:-1]

    return messages


def add_authorized_key(conn: Connection, user, public_key):
    conn.sudo('mkdir -p /home/{0}/.ssh'.format(user), user=user)
    conn.sudo('touch /home/{0}/.ssh/authorized_keys'.format(user), user=user)
    conn.sudo('chmod -R go= /home/{0}/.ssh'.format(user), user=user)
    conn.sudo('echo "{0}" | sudo tee -a /home/{1}/.ssh/authorized_keys'
              .format(public_key, user), pty=True)


def install_packages(conn: Connection, packages: Iterable[str]):
    apt = "DEBIAN_FRONTEND=noninteractive apt-get install -y {}"
    for package in packages:
        conn.sudo(apt.format(package))



from typing import Optional
from fabric.connection import Connection


def ensure_directory(conn: Connection,
                     directory: str,
                     owning_group: str,
                     mod: str = 'ug+rwX,o+rX,o-w'):
    conn.sudo('mkdir -p {0}'.format(directory))
    set_permissions_directory(conn, directory, group=owning_group, mod=mod)


def set_permissions_directory(conn: Connection,
                              directory: str,
                              group: Optional[str] = None,
                              user: Optional[str] = None,
                              mod: str = '660',
                              setgid: bool = True):
    if group is not None:
        conn.sudo('chgrp -R {0} {1}'.format(group, directory))

    if user is not None:
        conn.sudo('chown -R {0} {1}'.format(user, directory))

    conn.sudo('chmod -R {0} {1}'.format(mod, directory))

    if setgid:
        conn.sudo('chmod -R g+s {0}'.format(directory))


def set_permissions_file(conn: Connection,
                         file: str,
                         user: Optional[str] = None,
                         group: Optional[str] = None,
                         mod: str = '644'):
    if group is not None:
        conn.sudo('chgrp {0} {1}'.format(group, file))

    if user is not None:
        conn.sudo('chown {0} {1}'.format(user, file))

    conn.sudo('chmod {0} {1}'.format(mod, file))
